"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from shlex import split
from subprocess import CompletedProcess, run

from Setter.Logging import Logging


class Command:
	"""
	Command execution class.

	Methods
	-------
	execute(command: str, require_sudo: bool = False, timeout: int = 60, std_input: str = None) -> CompletedProcess:
		Execute command and returns completed process.
	restart_service(service: str) -> CompletedProcess:
		Restart specified systemd service.
	"""

	@staticmethod
	def execute(command: str, require_sudo: bool = False, timeout: int = 60, std_input: str = None) -> CompletedProcess:
		"""
		Execute command and returns completed process.

		Command is split into tokens. Quotes are respected when processing command.

		Parameters
		----------
		command: str
			command to execute
		require_sudo: bool
			command requires sudo
		timeout: int
			command timeout
		std_input: str
			stdin for command

		Returns
		-------
		CompletedProcess
			completed process object with return code, stdout and stderr
		"""
		cmd = split(command)
		if require_sudo:
			cmd.insert(0, 'sudo')
		logger: Logger = Logging.get_logger()
		logger.debug('Running command: %s with timeout %d', command, timeout)
		process = run(args=cmd, timeout=timeout, input=std_input, capture_output=True, text=True, check=False)
		logger.debug('Command %s was executed.', command)
		logger.debug('Exit code: %d', process.returncode)
		logger.debug('Standard output: %s', process.stdout)
		logger.debug('Error output: %s', process.stderr)
		return process

	@staticmethod
	def restart_service(service: str) -> CompletedProcess:
		"""
		Restart specified systemd service.

		Parameters
		----------
		service: str
			service name

		Returns
		-------
		CompletedProcess
			completed process object with return code, stdout and stderr
		"""
		timeout = 120 if service == 'iqrf-gateway-daemon' else 60
		command = 'systemctl restart ' + service + '.service'
		return Command.execute(command, True, timeout)
