"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
from uuid import uuid4
from Setter.Exceptions.DaemonApiErrorException import DaemonApiErrorException
from Setter.Exceptions.WebsocketErrorException import WebsocketErrorException
from Setter.Logging import Logging
from Setter.WebsocketClient import WebsocketClient


class DaemonApi:
	"""
	Daemon API request and response handling class.

	Methods
	-------
	read_os(address: int) -> dict:
		Construct and send OS read request, then return response.
	restart_os(address: int) -> dict:
		Construct and send OS restart request, then return response.
	read_eeprom(address:int, start_addr: int, length: int) -> dict:
		Construct and send EEPROM read request, then return response.
	write_tr_config(address: int, args: dict) -> dict:
		Construct and send IQMESH write tr configuration request, then return response.
	send_request(request: dict) -> dict:
		Sends request to Daemon API.
	handle_response(response: str) -> dict:
		Handles and returns Daemon API response.
	"""

	__logger = Logging.get_logger()

	@classmethod
	def read_os(cls, address: int) -> dict:
		"""
		Construct and send OS read request, then return response.

		Parameters
		----------
		address: int
			device address

		Returns
		-------
		dict
			daemon api response
		"""
		cls.__logger.info('Sending OS read request.')
		request = {
			'mType': 'iqrfEmbedOs_Read',
			'data': {
				'msgId': str(uuid4()),
				'req': {
					'nAdr': address,
					'param': {}
				},
				'returnVerbose': True
			}
		}
		return cls.send_request(request)

	@classmethod
	def restart_os(cls, address: int) -> dict:
		"""
		Construct and send OS restart request, then return response.

		Parameters
		----------
		address: int
			device address

		Returns
		-------
		dict
			daemon api response
		"""
		cls.__logger.info('Sending OS restart request.')
		request = {
			'mType': 'iqrfEmbedOs_Restart',
			'data': {
				'msgId': str(uuid4()),
				'req': {
					'nAdr': address,
					'param': {}
				},
				'returnVerbose': True
			}
		}
		return cls.send_request(request)

	@classmethod
	def read_eeprom(cls, address: int, start_addr: int, length: int) -> dict:
		"""
		Construct and send EEPROM read request, then return response.

		Parameters
		----------
		address: int
			device address
		start_addr: int
			address to read from
		length: int
			number of bytes to read

		Returns
		-------
		dict
			daemon api response
		"""
		cls.__logger.info('Sending EEPROM read request')
		request = {
			'mType': 'iqrfEmbedEeprom_Read',
			'data': {
				'msgId': str(uuid4()),
				'req': {
					'nAdr': address,
					'param': {
						'address': start_addr,
						'len': length
					}
				},
				'returnVerbose': True
			}
		}
		return cls.send_request(request)

	@classmethod
	def write_tr_config(cls, address: int, args: dict) -> dict:
		"""
		Construct and send IQMESH write tr configuration request, then return response.

		Parameters
		----------
		address: int
			device address
		args: dict
			dictionary of write tr configuration request parameters

		Returns
		-------
		dict
			daemon api response
		"""
		cls.__logger.info('Sending OS WriteTrConf request')
		request = {
			'mType': 'iqmeshNetwork_WriteTrConf',
			'data': {
				'msgId': str(uuid4()),
				'req': {
					'deviceAddr': address
				},
				'returnVerbose': True
			}
		}
		for key, val in args.items():
			request['data']['req'][key] = val
		return cls.send_request(request)

	@classmethod
	def send_request(cls, request: dict) -> dict:
		"""
		Send request to Daemon API.

		Parameters
		----------
		request: dict
			daemon api request

		Returns
		-------
		dict
			daemon api response

		Exceptions
		----------
		WebsocketErrorException
			raised if an error occured while sending request or receiving response
		"""
		for i in range(5):
			WebsocketClient.send(json.dumps(request))
			try:
				response = WebsocketClient.receive()
				return cls.handle_response(response)
			except WebsocketErrorException as ws_error:
				if i < 4:
					continue
				raise ws_error

	@classmethod
	def handle_response(cls, response: str) -> dict:
		"""
		Handle and returns Daemon API response.

		Parameters
		----------
		response: str
			daemon api response string

		Returns
		-------
		dict
			daemon api response

		Exceptions
		----------
		DaemonApiErrorException
			raised if request ended with errorous status code
		"""
		json_rsp = json.loads(response)
		if json_rsp['data']['status'] != 0:
			error_str = Logging.daemon_api_error(json_rsp['data']['statusStr'])
			cls.__logger.error(error_str)
			raise DaemonApiErrorException(error_str)

		return json_rsp
