"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import shutil
from logging import Logger
from pwd import getpwnam
from subprocess import CompletedProcess

from Setter.Common.AuxiliaryFunctions import AuxiliaryFunctions
from Setter.Common.Command import Command
from Setter.Exceptions.CommandException import CommandException
from Setter.Exceptions.SystemErrorException import SystemErrorException
from Setter.Logging import Logging


class WebappConfig:
	"""
	IQRF Gateway Webapp configuration class.

	Methods
	-------
	configure(config: dict) -> None:
		Configure webapp database, nginx and clear cache.
	clear_cache() -> None:
		Clear webapp cache.
	configure_access(password: str) -> None:
		Configure access password.
	configure_database() -> None:
		Create database file, apply migrations and import patches.
	nginx_default_site(hostname: str) -> None:
		Create nginx available sites symlinks and set enabled site.
	"""

	__name: str = 'IQRF Gateway Webapp'
	__conf_path: str = '/etc/iqrf-gateway-webapp/nginx/'
	__site_path: str = '/etc/nginx/sites-available/iqrf-gateway-webapp.localhost'
	__site_path_https: str = '/etc/nginx/sites-available/iqrf-gateway-webapp-https.localhost'
	__logger: Logger = Logging.get_logger()

	@classmethod
	def configure(cls, config: dict) -> None:
		"""
		Configure webapp database, nginx and clear cache.

		Parameters
		----------
		config: dict
			gateway configuration file

		Returns
		-------
		None
		"""
		cls.clear_cache()
		cls.configure_database()
		cls.nginx_default_site(config['gwHost'])

	@classmethod
	def configure_admin_account(cls, password: str) -> None:
		"""
		Configure access password.

		Parameters
		----------
		password: str
			Password

		Returns
		-------
		None
		"""
		result = WebappConfig.run_manager_command(f'user:add -u admin -p {password} -r admin -l en')
		if result.returncode != 0:
			error_str = f'Failed to create webapp user: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)

	@classmethod
	def clear_cache(cls) -> None:
		"""
		Clear webapp cache.

		Parameters
		----------
		None

		Returns
		-------
		None

		Exceptions
		----------
		CommandException
			raised if webapp cache clear command fails
		"""
		cls.__logger.info('Clearing %s cache.', cls.__name)
		result = cls.run_manager_command('nette:cache:purge')
		if result.returncode != 0:
			error_str = f'Failed to clear webapp cache: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)

	@classmethod
	def configure_database(cls) -> None:
		"""
		Create database file, apply migrations and import patches.

		Parameters
		----------
		None

		Returns
		-------
		None

		Exceptions
		----------
		CommandException
			raised if webapp db creation, migrations or patch importing fails
		"""
		cls.__logger.info('Configuring %s.', cls.__name)
		if os.path.exists('/var/lib/iqrf-gateway-webapp/database.db'):
			cls.__logger.info('Webapp database already exists.')
			return
		if os.path.exists('/usr/share/iqrf-gateway-webapp/app/config/database.db'):
			cls.__logger.info('Webapp database exists in "/usr/share", fixing.')
			shutil.move('/usr/share/iqrf-gateway-webapp/app/config/database.db', '/var/lib/iqrf-gateway-webapp/database.db')
			user = getpwnam('www-data')
			os.chown('/var/lib/iqrf-gateway-webapp/database.db', user[2], user[3])
			return
		result = cls.run_manager_command('database:create')
		if result.returncode != 0:
			error_str = f'Failed to create webapp database: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)
		result = cls.run_manager_command('migrations:migrate --no-interaction')
		if result.returncode != 0:
			error_str = f'Failed to execute migrations: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)
		result = cls.run_manager_command('doctrine:fixtures:load --append --no-interaction')
		if result.returncode != 0:
			error_str = f'Failed to load fixtures: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)
		result = cls.run_manager_command('iqrf-os:import-patches')
		if result.returncode != 0:
			error_str = f'Failed to import IQRF OS patches to database: {result.stderr}.'
			cls.__logger.error(error_str)
			raise CommandException(error_str)

	@classmethod
	def nginx_default_site(cls, hostname) -> None:
		"""
		Create nginx available sites symlinks.

		If hostname indicates that the gateway is iqaros system, iqaros nginx configuration is used.

		Parameters
		----------
		hostname: str
			gateway hostname

		Returns
		-------
		None

		Exceptions
		----------
		SystemErrorException
			raised if symlinks cannot be created
		"""
		cls.__logger.info('Configuring default site for %s.', cls.__name)
		try:
			if 'iqaros' in hostname:
				AuxiliaryFunctions.symlink_overwrite(cls.__conf_path + 'iqrf-gateway-webapp-iqaros.localhost', cls.__site_path)
				AuxiliaryFunctions.symlink_overwrite(cls.__conf_path + 'iqrf-gateway-webapp-iqaros-https.localhost', cls.__site_path_https)
			else:
				AuxiliaryFunctions.symlink_overwrite(cls.__conf_path + 'iqrf-gateway-webapp.localhost', cls.__site_path)
				AuxiliaryFunctions.symlink_overwrite(cls.__conf_path + 'iqrf-gateway-webapp-https.localhost', cls.__site_path_https)
		except OSError as os_error:
			error_str = f'Failed to configure nginx default site: {os_error.strerror} - {os_error.filename}.'
			cls.__logger.error(error_str)
			raise SystemErrorException(error_str) from os_error

	@staticmethod
	def run_manager_command(command: str) -> CompletedProcess:
		"""
		Run webapp manager command.

		Parameters
		----------
		command: str
			Webapp manager command to execute
		Returns
		-------
		CompletedProcess
			completed process object with return code, stdout and stderr
		"""
		return Command.execute(f'su - www-data -s /bin/bash -c "/usr/sbin/iqrf-gateway-webapp-manager {command}"')
