"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time
from logging import Logger
import websocket
from Setter.Exceptions.WebsocketErrorException import WebsocketErrorException
from Setter.Logging import Logging


class WebsocketClient:
	"""
	Websocket client implementation class.

	Methods
	-------
	create_connection(broker: str, port: int) -> None:
		Establish connection to websocket server at specified hostname and port.
	send(payload: str) -> None:
		Send message to websocket server.
	receive() -> str:
		Receive message from websocket server.
	"""

	__connection = None
	__logger: Logger = Logging.get_logger()

	@classmethod
	def create_connection(cls, host: str, port: int) -> None:
		"""
		Establish connection to websocket server at specified hostname and port.

		Parameters
		----------
		host: str
			server hostname
		port: int
			server port

		Returns
		-------
		None

		Exceptions
		----------
		WebsocketErrorException
			raised if connection cannot be established
		"""
		url = 'ws://' + host + ':' + str(port)
		for i in range(10):
			try:
				cls.__connection = websocket.create_connection(url, 10)
				cls.__logger.info('Connected to websocket.')
				break
			except Exception as ws_error:
				time.sleep(3)
				if i < 9:
					continue
				error_str = f'Failed to open websocket connection to IQRF Gateway Daemon: {str(ws_error)}.'
				cls.__logger.error(error_str)
				raise WebsocketErrorException(error_str) from ws_error

	@classmethod
	def send(cls, payload: str) -> None:
		"""
		Send message to websocket server.

		Parameters
		----------
		payload: str
			message to send

		Returns
		-------
		None
		"""
		cls.__logger.info('Sending via websocket: %s.', payload)
		cls.__connection.settimeout(10)
		cls.__connection.send(payload)

	@classmethod
	def receive(cls) -> str:
		"""
		Receive message from websocket server.

		Parameters
		----------
		None

		Returns
		-------
		str
			received message

		Exceptions
		----------
		WebsocketErrorException
			raised if an error occurs while retrieving message
		"""
		try:
			payload = cls.__connection.recv()
			cls.__logger.info('Received via websocket: %s.', payload)
			return payload
		except Exception as ws_error:
			cls.__logger.error(str(ws_error))
			raise WebsocketErrorException(str(ws_error)) from ws_error

	@classmethod
	def close_connection(cls) -> None:
		"""
		Close websocket connection.

		Parameters
		----------
		None

		Returns
		-------
		None
		"""
		cls.__connection.close()
		cls.__connection = None
